! WHIZARD 1.51 Jun 15 2005
! 
! (C) 1999-2004 by Wolfgang Kilian <kilian@particle.uni-karlsruhe.de>
!
! WHIZARD is free software; you can redistribute it and/or modify it
! under the terms of the GNU General Public License as published by 
! the Free Software Foundation; either version 2, or (at your option)
! any later version.
!
! WHIZARD is distributed in the hope that it will be useful, but
! WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this program; if not, write to the Free Software
! Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! This file has been stripped of most comments.  For documentation, refer
! to the source 'whizard.nw'

module pdflib_interface

  use kinds, only: default !NODEP!
  use kinds, only: double !NODEP!
  use diagnostics, only: msg_fatal, msg_warning
  use user, only: PDF_user_init => PDF_init
  use user, only: PDF_user_strfun_array => PDF_strfun_array

  implicit none
  private

  public :: pdflib_init, pdflib_strfun_array, lhapdflib_init, &
        lhapdflib_strfun_array
 
  integer, parameter :: TQUARK = 6, GLUON1 = 9, GLUON2 = 21
  integer, parameter :: PROTON = 2212, ANTIPROTON = -PROTON, PHOTON = 22, &
        PION = 211, ANTIPION = -PION

  integer, parameter, public :: PDFSET_ARRAY_SIZE = 20
  integer, parameter, public :: PDFSET_CHAR_LEN = 20

  logical, save :: pdf_invert = .false.
  logical, save :: pdf_user = .false.

contains

  subroutine pdflib_init &
       & (pdg_code_in, ngroup, nset, nfl, lo, QCDL4, top_mass, first)
    integer, intent(in) :: pdg_code_in, ngroup, nset, nfl, lo
    real(kind=default), intent(in) :: QCDL4, top_mass
    logical, intent(in), optional :: first
    integer :: nptype, i
    character (len=PDFSET_CHAR_LEN), dimension(PDFSET_ARRAY_SIZE) :: parm
    real(kind=double), dimension(PDFSET_ARRAY_SIZE) :: value
    real(kind=double), parameter :: x_dummy = 0.5_double
    real(kind=double), parameter :: scale_dummy = 100._double
    real(kind=double), dimension(-TQUARK:TQUARK) :: dxpdf_dummy
    external pftopdg
    select case (pdg_code_in)
    case (PROTON)
       nptype = 1
       pdf_invert = .false.
    case (ANTIPROTON)
       nptype = 1
       pdf_invert = .true.           
    case (PHOTON)
       nptype = 3
       pdf_invert = .false.
    case default
       call msg_fatal &
            & (" PDFlib called for beam different from (anti)proton or photon")
    end select
    if (ngroup < 0) then
       pdf_user = .true.
       call PDF_user_init &
            & (pdg_code_in, abs(ngroup), nset, nfl, lo, &
            &  real(QCDL4,kind=double), real(top_mass,kind=double))
    else
       pdf_user = .false.
       parm = " "
       value = 0
       i = 1
       if (nptype > 0) then
          parm (i) = "NPTYPE";  value(i) = nptype;   i = i+1
       end if
       if (ngroup > 0) then
          parm (i) = "NGROUP";  value(i) = ngroup;   i = i+1
       end if
       if (nset > 0) then
          parm (i) = "NSET";    value(i) = nset;     i = i+1
       end if
       if (nfl > 0) then
          parm (i) = "NFL";     value(i) = nfl;      i = i+1
       end if
       if (lo > 0) then
          parm (i) = "LO";      value(i) = lo;       i = i+1
       end if
       if (QCDL4 > 0) then
          parm (i) = "QCDL4";   value(i) = QCDL4;    i = i+1
       end if
       if (top_mass > 0) then
          parm (i) = "TMAS";    value(i) = top_mass; i = i+1
       end if
       if (present(first)) then
          if (first)  call pftopdg (x_dummy, scale_dummy, dxpdf_dummy)
       end if
    end if
  end subroutine pdflib_init

  subroutine lhapdflib_init &
       & (pdg_code_in, file, set, first)
    integer, intent(in) :: pdg_code_in, set
    character(len=PDFSET_CHAR_LEN) :: file
    logical, intent(in), optional :: first
    real(kind=double), parameter :: x_dummy = 0.5_double
    real(kind=double), parameter :: scale_dummy = 100._double
    real(kind=double), dimension(-TQUARK:TQUARK) :: dxpdf_dummy
    select case (pdg_code_in)
    case (PROTON)
       pdf_invert = .false.
    case (ANTIPROTON)
       pdf_invert = .true.
    case (PHOTON)
       pdf_invert = .false.
    case (PION)
       pdf_invert = .false.
    case (ANTIPION) 
       pdf_invert = .true.
    case default
       call msg_fatal &
            & (" LHAPDF only works for p, pbar, (anti-)pion, photon")
    end select
    call InitPDFsetByName (file)        
    call InitPDF (set)
    if (present(first)) then
       if (first) call & 
          & lha_pftopdg (pdg_code_in, x_dummy, scale_dummy, dxpdf_dummy)
    end if
    contains
        subroutine lha_pftopdg (pdg_code_in, xdum, scaldum, dxpdfdum)
        integer, intent(in) :: pdg_code_in
        real(kind=double), intent(in) :: xdum 
        real(kind=double), intent(in) :: scaldum 
        real(kind=double), dimension(-TQUARK:TQUARK) :: dxpdfdum, f
        select case (pdg_code_in)
        case (PROTON,ANTIPROTON,PION,ANTIPION)
           call evolvePDF (xdum, scaldum, f)       
        case (PHOTON)
           call evolvePDFp (xdum, scaldum, 0.0_double, 0, f)
        case default 
           call msg_fatal &
              & (" No structure function available. ")
        end select
        dxpdfdum(0) = f(0)
        dxpdfdum(-2:2) = f(-2:2)
        dxpdfdum(3:6) = f(3:6)
        dxpdfdum(-3:-6) = f(-3:-6)
        end subroutine lha_pftopdg      
  end subroutine lhapdflib_init

  subroutine pdflib_strfun_array (x, scale, rho)
    real(kind=default), intent(in) :: x, scale
    real(kind=default), dimension(-TQUARK:TQUARK), intent(out) :: rho
    real(kind=double), dimension(-TQUARK:TQUARK) :: dxpdf
    external pftopdg
    if (pdf_user) then
       call PDF_user_strfun_array &
            & (real(x,kind=double), real(scale,kind=double), dxpdf)
    else
       call pftopdg (real(x,kind=double), real(scale,kind=double), dxpdf)
    end if
    if (.not.pdf_invert) then
       rho = dxpdf / x
    else
       rho = dxpdf(TQUARK:-TQUARK:-1) / x
    end if
  end subroutine pdflib_strfun_array

  subroutine lhapdflib_strfun_array (pdg_code_in, set, x, scale, rho)
    integer, intent(in) :: pdg_code_in, set     
    real(kind=default), intent(in) :: x, scale
    real(kind=default) :: xmin, xmax, q2min, q2max
    real(kind=default), dimension(-TQUARK:TQUARK), intent(out) :: rho
    real(kind=double), dimension(-TQUARK:TQUARK) :: dxpdf
    if (pdf_user) then
       call PDF_user_strfun_array &
            & (real(x,kind=double), real(scale,kind=double), dxpdf)
    else
       call GetMinMax (set,xmin,xmax,q2min,q2max)
       if (x .ge. xmin .and. x .le. xmax .and. scale**2 .ge. &
          & q2min .and. scale**2 .le. q2max) then
            select case (abs (pdg_code_in))
            case (PROTON, PION)
               call evolvePDF (x, scale, dxpdf)    
            case (PHOTON)
               call evolvePDFp (x, scale, 0.0_double, 0, dxpdf)
            end select
       else 
          dxpdf = 0
       end if   
    end if 
    !!! Here we catch negatively running sea quark densities for some of 
    !!! the .LHpdf files in LHAPDF
    dxpdf = max (dxpdf,0.0_default)
    if (.not.pdf_invert) then
       rho = dxpdf / x
    else
       rho = dxpdf(TQUARK:-TQUARK:-1) / x
    end if
  end subroutine lhapdflib_strfun_array

end module pdflib_interface
